from unittest import result
import mugrade
import numpy as np

def objects_equal(value,ref):
    """ Test if two objects are equal according to our autograding rules.
        We have to do this manually so that numpy arrays are compared properly.
    """

    try:
        if isinstance(ref, dict):
            if sorted(value.keys()) != sorted(ref.keys()):
                return False
            for k,v in ref.items():
                if not objects_equal(v, value[k]):
                    return False
            return True
    
        elif isinstance(ref, list) or isinstance(ref, tuple):
            if len(value) != len(ref):
                return False
            for a,b in zip(ref, value):
                if not objects_equal(a, b):
                    return False
            return True
    
        elif isinstance(ref, np.ndarray):
            if value.shape != ref.shape:
                return False
            if not np.allclose(ref, value, atol=1e-8):
                return False
            return True
          
        elif isinstance(ref, float) or isinstance(ref, np.float32):
          return np.allclose(value, ref)
    
        else:
            return value == ref
    except:
        return False

def test_Graph():
    simple_graph1 = Graph().add_edges([("A","B"), ("B","C"), ("A","D"), ("D", "E"), ("E", "B")])
    with mugrade.test:
        assert simple_graph1.edges == {'A': {'B': 1.0, 'D': 1.0}, 'B': {'C': 1.0}, 'C': {}, 'D': {'E': 1.0}, 'E': {'B': 1.0}}
    
    wikipedia_edges = read_graph()
    wikipedia_graph = Graph().add_edges(wikipedia_edges)
    with mugrade.test:
        assert len(wikipedia_graph.edges) == 24166
    with mugrade.test:
        assert len(wikipedia_graph.edges["Carnegie_Mellon_University"]) == 162

def submit_Graph():
    wikipedia_edges = read_graph()
    wikipedia_graph = Graph().add_edges(wikipedia_edges)
    mugrade.submit(len(wikipedia_graph.edges["United_States"]))

    simple_graph2 = Graph().add_edges([("A","B"), ("B","C"), ("A","D"), ("D", "E")])
    mugrade.submit(simple_graph2.edges)

def test_shortest_path():
    simple_graph1 = Graph().add_edges([("A","B"), ("B","C"), ("A","D"), ("D", "E"), ("E", "B")])
    with mugrade.test:
        assert shortest_path(simple_graph1, "A") == {'A': (0, None), 'B': (1, 'A'), 'C': (2, 'B'), 'D': (1, 'A'), 'E': (2, 'D')}
    with mugrade.test:
        assert shortest_path(simple_graph1, "B") == {'A': (float("inf"), None), 'B': (0, None), 'C': (1, 'B'), 'D': (float("inf"), None), 'E': (float("inf"), None)}

    wikipedia_edges = read_graph()
    wikipedia_graph = Graph().add_edges(wikipedia_edges)
    shortest_paths_cmu = shortest_path(wikipedia_graph, "Carnegie_Mellon_University")
    with mugrade.test:
        assert shortest_paths_cmu["GNU_Free_Documentation_License"] == (2, 'Wikimedia_Commons')

def submit_shortest_path():
    simple_graph1 = Graph().add_edges([("A","B"), ("B","C"), ("A","D"), ("D", "E"), ("E", "B")])
    mugrade.submit(shortest_path(simple_graph1, "D"))

    wikipedia_edges = read_graph()
    wikipedia_graph = Graph().add_edges(wikipedia_edges)
    shortest_paths_cmu = shortest_path(wikipedia_graph, "Carnegie_Mellon_University")
    mugrade.submit(shortest_paths_cmu["List_of_Salticidae_species"])
    mugrade.submit(max(v[0] for k,v in shortest_paths_cmu.items() if v[0] != float("inf")))

def test_adjacency_matrix():
    simple_graph1 = Graph().add_edges([("A","B"), ("B","C"), ("A","D"), ("D", "E"), ("E", "B")])
    with mugrade.test:
        assert np.all(adjacency_matrix(simple_graph1)[0].row == [1, 3, 2, 4, 1])
    with mugrade.test:
        assert np.all(adjacency_matrix(simple_graph1)[0].col == [0, 0, 1, 3, 4])

    wikipedia_edges = read_graph()
    wikipedia_graph = Graph().add_edges(wikipedia_edges)
    with mugrade.test:
        assert adjacency_matrix(wikipedia_graph)[0].nnz == 6081246
    with mugrade.test:
        assert np.all(np.asarray((lambda g, M, n: M.tocsr()[len(g.edges["Andrew_Carnegie"])*[list(n).index("Andrew_Carnegie")],[list(n).index(e) for e in sorted(g.edges["Andrew_Carnegie"])]])(*(lambda g : (g,) + adjacency_matrix(g))(wikipedia_graph)))[0].tolist() == [0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 1.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 1.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0])

def submit_adjacency_matrix():
    wikipedia_edges = read_graph()
    wikipedia_graph = Graph().add_edges(wikipedia_edges)
    mugrade.submit(np.asarray((lambda g, M, n: M.tocsr()[len(g.edges["Carnegie_Mellon_University"])*[list(n).index("Carnegie_Mellon_University")],[list(n).index(e) for e in sorted(g.edges["Carnegie_Mellon_University"])]])(*(lambda g : (g,) + adjacency_matrix(g))(wikipedia_graph)))[0])
    mugrade.submit(list(sorted(adjacency_matrix(wikipedia_graph)[1]))[40:50])

def test_pagerank():
    simple_graph1 = Graph().add_edges([("A","B"), ("B","C"), ("A","D"), ("D", "E"), ("E", "B")])
    with mugrade.test:
        expected_result = {'A': 0.08510862387068166, 
                           'B': 0.28124676686965944, 
                           'C': 0.3241683757098922, 
                           'D': 0.12127978901572137, 
                           'E': 0.18819644453404483}
        result = pagerank(simple_graph1, 0.85, 100)
        assert objects_equal(result, expected_result)
    with mugrade.test:
        expected_result = {'A': 0.08031128654665567, 
                           'B': 0.28305712943368583, 
                           'C': 0.33506270303695607, 
                           'D': 0.11645136549265456, 
                           'E': 0.18511751549004807}
        result = pagerank(simple_graph1, 0.9, 50)
        assert objects_equal(result, expected_result)
    wikipedia_edges = read_graph()
    wikipedia_graph = Graph().add_edges(wikipedia_edges)
    with mugrade.test:
        expected_result = {'United_States': np.array(0.00275189),
                           '2007': np.array(0.00244339), 
                           '2008': np.array(0.00217343), 
                           'Wikimedia_Commons': np.array(0.00172604), 
                           'United_Kingdom': np.array(0.00159999), 
                           '2006': np.array(0.00154845), 
                           'France': np.array(0.00144959), 
                           'Wiktionary': np.array(0.00126753), 
                           'Canada': np.array(0.00109896), 
                           'World_War_II': np.array(0.00104913), 
                           '2005': np.array(0.00104633), 
                           'List_of_African_films': np.array(0.00100714), 
                           'Germany': np.array(0.00095626), 
                           'Europe': np.array(0.00093769), 
                           'English_language': np.array(0.00090814), 
                           'Geographic_coordinate_system': np.array(0.00089171), 
                           'Latin': np.array(0.00088866), 
                           'Australia': np.array(0.00087985), 
                           'India': np.array(0.00078763), 
                           'Japan': np.array(0.00078482)}
        result = {k: np.array(v) for k,v in sorted(pagerank(wikipedia_graph, 0.85, 100).items(), key = lambda x : x[1], reverse=True)[:20]}
        assert objects_equal(result, expected_result)

def submit_pagerank():
    simple_graph1 = Graph().add_edges([("A","B"), ("B","C"), ("A","D"), ("D", "E"), ("E", "B")])
    mugrade.submit(pagerank(simple_graph1, 0.75, 20))
    mugrade.submit(pagerank(simple_graph1, 0.6, 100))

    wikipedia_edges = read_graph()
    wikipedia_graph = Graph().add_edges(wikipedia_edges)
    mugrade.submit({k: np.array(v) for k,v in sorted(pagerank(wikipedia_graph, 0.85, 100).items(), key = lambda x : x[1], reverse=True)[20:40]})
